/* Copyright (c) 2002 Petr Bilek
   Tento kod je volne programove vybaveni; muzete jej sirit a modifikovat
podle ustanoveni Obecne verejne license GNU, vydavane FreeSoftware Foundation.
Na tento program se nevztahuji ZADNE ZARUKY; neposkytuji se ani odvozene zaruky
PRODEJNSTI anebo VHODNOSTI PRO URCITY UCEL. */
/* soubor vlakna.c */

#ifndef _REENTRANT
#define _REENTRANT
#endif
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <time.h>
#include <errno.h>
#include <signal.h>

#include "struktury.h"
#include "lib.h"
#include "vlakna.h"

#ifdef PSANIPROFI
#include "thread-ncurses.h"
#endif

static pthread_t vlakno[POCETVLAKEN];
static boolean vlspusteno[POCETVLAKEN];

#ifdef PSANIPROFI
static void nastavSignal()
{
    struct sigaction sa;
    sigset_t set;
    sigemptyset(&set);
    sa.sa_handler = thncurses_signal;
    sa.sa_mask = set;
    sa.sa_flags = SA_RESTART;
    sa.sa_restorer = NULL;
    sigaction(SIGWINCH, &sa, NULL);
}

static void *winch_thread(void *arg)
{
    sigset_t set;
    int i;
    i = pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
    if (i && (global_get(g_warning) == g_w_ano)) {
        print_hl(1,
                 "winch_thread: Selhalo nastaveni priznaku ukonceni funkce varovani");
        return NULL;
    }

    sigemptyset(&set);
    sigaddset(&set, SIGWINCH);
    sigprocmask(SIG_UNBLOCK, &set, NULL);
    nastavSignal();
    while (true) {
        sleep(1000);
        pthread_testcancel();
    }
    return NULL;
}
#endif

void vlakna(type_uziv * slozka, type_nastaveni * nastaveni)
{
    int res;
    pthread_attr_t vlakno_atr;

    char *zprava[] = { "                   V A R O V A N I:\n"
            "  Vas operacni system nepodporuje novy standard POSIX\n"
            "        Program nejspis nebude pracovat spravne!\n",
        "                        < OK >"
    };
    char *zprava2[] = { "                   V A R O V A N I:\n"
            "           Vas system nepodporuje POSIX.1c\n\n"
            "                        < 0K >"
    };

#ifdef _POSIX_VERSION
    if (_POSIX_VERSION < 199506L)
#endif
        if (global_get(g_warning) == g_w_ano)
            (void) button(zprava, 2, 1, 6, 55, 10, 14);


#if defined _POSIX_C_SOURCE && defined _POSIX_VERSION
    if (_POSIX_VERSION < 199506L && _POSIX_C_SOURCE <= 199506L)
#endif
        if (global_get(g_warning) == g_w_ano) {
            (void) button(zprava2, 2, 1, 6, 55, 10, 14);
        }

    res = pthread_attr_init(&vlakno_atr);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1, "Chyba inicializace vlaken --> ukonete program!");
        return;
    }

    /* zablokuje SIGWINCH pro toto vlkno i vechna nov vytvoen vlnka */
    sigset_t set;
    sigemptyset(&set);
    sigaddset(&set, SIGWINCH);
    sigprocmask(SIG_BLOCK, &set, NULL);
/* --------------- spousteni vlaken -------------- */
    res =
        pthread_create(&vlakno[0], &vlakno_atr, vl_metronom,
                       (void *) nastaveni);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1, "Chyba spoutn metronomu --> ukonete program!");
        vlspusteno[0] = 0;
        return;
    } else
        vlspusteno[0] = 1;

    res = pthread_create(&vlakno[1], &vlakno_atr, vl_napoveda3, NULL);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1,
                     "Chyba spoutn \"napovedy3\" --> ukonete program!");
        vlspusteno[1] = 0;
        return;
    } else
        vlspusteno[1] = 1;

    vlspusteno[2] = 0;
    if (global_get(g_disable_inet) != g_w_ano) {
        res =
            pthread_create(&vlakno[2], &vlakno_atr, vl_http_version, NULL);
        if (res != 0) {
            if (global_get(g_warning) == g_w_ano)
                print_hl(1, "Chyba spoutn \"vl_http_version\"");
        } else
            vlspusteno[2] = 1;
    }
#ifdef PSANIPROFI
    res = pthread_create(&vlakno[3], &vlakno_atr, vl_varovani, NULL);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1,
                     "Chyba spoutn \"varovani\" --> ukonete program!");
        vlspusteno[3] = 0;
        return;
    } else
        vlspusteno[3] = 1;

    res = pthread_create(&vlakno[4], &vlakno_atr, vl_obnov_datum, NULL);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1, "Chyba spoutn vlaken --> ukonete program!");
        vlspusteno[4] = 0;
        return;
    } else
        vlspusteno[4] = 1;


    static THNcursesJob tjob;   // musi byt static, aby to vlaknu nezmizelo pod rukama
    tjob.fce = zpracuj_winch_job;
    tjob.data = NULL;

    res =
        pthread_create(&vlakno[5], &vlakno_atr, thncurses_signal_thread,
                       (void *) &tjob);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1,
                     "Chyba spoutn \"thncurses_signal_thread\" --> ukonete program!");
        vlspusteno[5] = 0;
        return;
    } else
        vlspusteno[5] = 1;

    res = pthread_create(&vlakno[6], &vlakno_atr, winch_thread, NULL);
    if (res != 0) {
        if (global_get(g_warning) == g_w_ano)
            print_hl(1,
                     "Chyba spoutn \"winch_thread\" --> ukonete program!");
        vlspusteno[6] = 0;
        return;
    } else
        vlspusteno[6] = 1;

/*	res = pthread_create(&vlakno[6], &vlakno_atr, vl_sporic, NULL);

	if (res != 0) {
	   if (global_get(g_warning) == g_w_ano)
		print_hl(1,
			 "Chyba spoutn \"sporic\" --> ukonete program!");
	    vlspusteno[6] = 0;
	    return;
	}  else vlspusteno[6] = 1;*/
#endif

    (void) pthread_attr_destroy(&vlakno_atr);
}

void vlakna_konec(void)
{
    int i, res;
    char chyba[22];
    for (i = 0; i < POCETVLAKEN; i++) {
        if (!vlspusteno[i])
            continue;
        res = pthread_cancel(vlakno[i]);
        if (res && (global_get(g_warning) == g_w_ano)) {
            sprintf(chyba, "Ukonceni vlakna %2i", i);
            perror(chyba);
        }
    }
    for (i = 0; i < POCETVLAKEN; i++) {
        if (!vlspusteno[i])
            continue;
        if (global_get(g_warning) == g_w_ano)
            fprintf(stderr, "Cekam na konec vlakna %i/%i\n", i + 1,
                    POCETVLAKEN);
        pthread_join(vlakno[i], NULL);
    }
}
